/* class Mutex
 * encapsulates the windows Mutex
 * Copyright (C) 2002 Michel Leunen
 * http://www.leunen.com/
 * michel@leunen.com
 *
 * This software is freeware and is provided at no charge to the user.
 * This software is provided 'as-is', without any express or implied warranty.
 * To the maximum extent permitted by applicable law will the author NOT be held
 * liable for any damages whatsoever arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose, including
 * commercial applications, and to freely redistribute it, with the following
 * restrictions:
 *
 * 1. If you use this software in a product, an acknowledgment in the product
 *    documentation would be appreciated.
 *
 * 2. If you use this software in a commercial application, please send me an email
 *	  detailing the application.
 *
 * 3. This notice may not be removed or altered from any source distribution.
 *
 * 4. It may be distributed by any means, provided that the original files
 * 		as supplied by the author remain intact and no charge is made other than
 * 		for reasonable distribution costs. If you are a vendor, if you want to
 * 		include this software in a compilation or in a magazine cover disk
 * 		you HAVE TO CONTACT ME to obtain my authorization before distribution.
 */

#ifndef MUTEX_H
#define MUTEX_H

#include <windows.h>
//------------------------------------------------------------
namespace mlLib
{

class Mutex
{
  private:
    HANDLE MutexHandle_;
    bool AlreadyExists_;
    Mutex(const Mutex&);
    Mutex& operator=(const Mutex&);
  public:
    /*
     * If a mutex is created with a given name and a mutex with
     * this name already exists, Windows doesn't report an error,
     * instead CreateMutex() returns the same handle as the previously
     * created one. If the mutex already exists, GetLastError()
     * returns ERROR_ALREADY_EXISTS.
     * An alternative is to first open the mutex, if it doesn't exist
     * OpenMutex() returns NULL. If it's the case, we create it.
     *
     * Mutex(const std::string& name):AlreadyExists_(false),
     * 		MutexHandle_(OpenMutex(MUTEX_ALL_ACCESS,TRUE,name.c_str()))
     * {
     *		if(MutexHandle_ == NULL)
     *    MutexHandle_ = CreateMutex(NULL,TRUE,name.c_str());
     * 		else AlreadyExists_=true;
     * }
     *
     * AlreadyExists() yields true if the mutex already existed
     * when we tried to create it.
     */
    Mutex(const std::string& name):AlreadyExists_(false),
        MutexHandle_(CreateMutex(NULL,TRUE,name.c_str()))
    {
      if(GetLastError() == ERROR_ALREADY_EXISTS)
        AlreadyExists_ = true;
    }
    ~Mutex()
    {
      ReleaseMutex(MutexHandle_);
      CloseHandle(MutexHandle_);
    }
    bool AlreadyExists()const{return AlreadyExists_;}
    operator bool()const{return MutexHandle_!=HANDLE(NULL);}
    HANDLE GetHandle(){return MutexHandle_;}
};
//------------------------------------------------------------
} //end namspace mlLib
#endif
